
open Format
open Pp
open Misc
open Logic
open Vcg

let oblig = Queue.create ()


let split_one (loc,n,o) =
  (* normal oblig *)
  let cn = ref 0 in
  let l = ref [] in 
  let push_normal o = 
    incr cn; l := (loc,n ^ "_" ^ string_of_int !cn,o) :: !l 
  in
  (* fpi oblig *)
  let cf = ref 0 in
  let push_fpi o = incr cf; Queue.add (loc, n ^ "_" ^ string_of_int !cf, o) in
  let rec split_rec = function
    | _ -> assert false
  in
  split_rec o;
  List.rev !l

let split ol = List.flatten (List.map split_one ol)
  
let rec print_term fmt = function
  | Tconst (ConstFloat f) -> fprintf fmt "%s" f
  | Tconst _ -> assert false
  | Tvar id -> Ident.print fmt id
  | Tapp (id, tl) -> 
      fprintf fmt "(%s %a)" (Ident.string id) (print_list space print_term) tl
  | Tderef _ -> assert false

let rec print_pred fmt = function
  | Pfpi (t,f1,f2) -> 
      fprintf fmt "(fpi %a %s %s)" print_term t f1 f2
  | Papp (id, tl) -> 
      fprintf fmt "(%s %a)" (Ident.string id) (print_list space print_term) tl
  | _ -> assert false

let print_hyp fmt = function
  | Svar _ -> assert false
  | Spred (_, p) -> print_pred fmt p

let print_hyps = print_list space print_hyp

let print_obligation fmt (loc,s,o) =
  fprintf fmt "%% %s from %a@\n" s Loc.report loc;
  begin match o with
    | [], p -> 
	fprintf fmt "(@[%a@])" print_pred p
    | [h], p -> 
	fprintf fmt "(@[IMPLIES %a@ %a@])" print_hyp h print_pred p
    | hl, p -> 
	fprintf fmt "(@[IMPLIES (AND %a)@ %a@])" print_hyps hl print_pred p
  end;
  fprintf fmt "@.@."

let print_obligations fmt =
  Queue.iter (print_obligation fmt) oblig;
  fprintf fmt "@."

let output f =
  print_in_file ~margin:78 print_obligations (f ^ ".fpi")

let reset () = Queue.clear oblig
